; Example Program glcd04.a51
; Graphic LCD: Solomon LM6321 (64 Columns x 8 Rows x 8 Bits x 2 Panels)
; Download image from PC via RS232
; Chin-Shiuh Shieh
; 2000-03-23
;
; Ping Assignment
GLCD_DI		EQU		P2.0	; 1 for Data, 0 for Instruction
GLCD_RW		EQU		P2.1	; 1 for Read, 0 for Write
GLCD_E		EQU		P2.2	; LCD Enable, 1 for Read, 1->0 for Write
GLCD_CS1	EQU		P2.3	; 1 for Chip Select 1 (Left Panel)
GLCD_CS2	EQU		P2.4	; 1 for Chip Select 2 (Right Panel)
GLCD_RST	EQU		P2.5	; 0 for LCD Reset
GLCD_PORT	EQU		P1		; LCD Port
; Memory Allocation
X       	DATA 	30h		; X-coordinate, vertically from 0(top) to 63(down)
Y       	DATA 	31h		; Y-coordinate, horizontally from 0(left) to 127(right)
XP      	DATA 	32h		; X-Page, vertically from 10111000b(top) to 10111111b(down)
YA      	DATA 	33h		; Y-Address, horizontally from 01000000b(left) to 01111111b(right)
MASK    	DATA 	34h		; LCD data pattern
VALUE   	DATA 	35h		; LCD data
IMGTF		BIT		00h		; Image Transfering Flag, 1 for transfering
;
PROG	EQU		0000h
		ORG		PROG+0000h
		SJMP	START
;
;Interrupt Vector Table
		ORG		PROG+0003h
		LCALL	INT0_ISR
		RETI
		ORG		PROG+000Bh
		LCALL	T0_ISR
		RETI
		ORG		PROG+0013h
		LCALL	INT1_ISR
		RETI
		ORG		PROG+001Bh
		LCALL	T1_ISR
		RETI
		ORG		PROG+0023h
		LCALL	UART_ISR
		RETI
;
		ORG		PROG+0030h
START:
        MOV     TMOD,#00100001b	; Timer1 in Mode 2, Timer0 in Mode 1
        MOV     TH1, #0FDh		; Baud Rate = 9600 bps at 11.0592MHz
        MOV     SCON,#01010000b	; UART in Mode 1
		SETB	ES				; Enable UART Interrupt
		SETB	EA				; Enable Interrupt
		SETB	TR1				; Start Timer 1
		CALL	GLCD_INIT		; Initialize Graphic LCD
		CLR		IMGTF
		MOV		X,#00h			; Initialize X coordinate
		MOV		Y,#00h			; Initialize Y coordinate
DONE:	SJMP	DONE
;
INT0_ISR:
		RET
T0_ISR:
		RET
INT1_ISR:
		RET
T1_ISR:
		RET
;
UART_ISR:
		JB		RI,UART_ISR_RI
UART_ISR_TI:
		CLR		TI
		RET
UART_ISR_RI:
		JB		IMGTF,PIXEL
		SETB	IMGTF
		SJMP	UART_ISR_RI_RET
		RET
PIXEL:
		MOV		A,SBUF
		CJNE	A,#00h,PIXEL_SET
PIXEL_CLR:
		CALL	GLCD_CLR
		SJMP	INC_Y
PIXEL_SET:
		CALL	GLCD_SET
INC_Y:
		INC		Y
		MOV		A,Y
		CJNE	A,#64,UART_ISR_RI_RET
		MOV		Y,#00h
INC_X:
		INC		X
		MOV		A,X
		CJNE	A,#64,UART_ISR_RI_RET
		MOV		X,#00h
		CLR		IMGTF
UART_ISR_RI_RET:
		MOV		A,SBUF
		MOV		SBUF,A			; Send Acknowledge
		CLR		RI
		RET
;
GLCD_WAIT:
		PUSH	ACC
		MOV		GLCD_PORT,#0FFh	; Reset GLCD_Port for input
		CLR		GLCD_DI
		SETB	GLCD_RW
GLCD_BUSY:
		SETB	GLCD_E
		MOV		A,GLCD_PORT
		CLR		GLCD_E
		JB		ACC.7,GLCD_BUSY
		POP		ACC
		RET
;
GLCD_WI:
		CALL	GLCD_WAIT
		CLR		GLCD_DI
		CLR		GLCD_RW
		MOV		GLCD_PORT,A
		SETB	GLCD_E
		CLR		GLCD_E
		RET
;
GLCD_WD:
		CALL	GLCD_WAIT
		SETB	GLCD_DI
		CLR		GLCD_RW
		MOV		GLCD_PORT,A
		SETB	GLCD_E
		CLR		GLCD_E
		RET
;
GLCD_RD:
        CALL    GLCD_WAIT
		MOV		GLCD_PORT,#0FFh		; Reset GLCD_Port for input
        SETB    GLCD_DI
		SETB	GLCD_RW
		SETB	GLCD_E
        MOV     A,GLCD_PORT
		CLR		GLCD_E
		RET
;
GLCD_INIT:
		CLR		GLCD_RST			; Reset LM6321
		SETB	GLCD_RST
		MOV		GLCD_PORT,#0FFh		; Reset GLCD_Port for input
; Reset left panel chip
		SETB	GLCD_CS1
		CLR		GLCD_DI
		SETB	GLCD_RW
GLCD_INIT1:
		SETB	GLCD_E
		MOV		A,GLCD_PORT
		CLR		GLCD_E
		JB		ACC.4,GLCD_INIT1	; Wait for left panel chip reset
		CLR		GLCD_CS1
; Reset right panel chip
		SETB	GLCD_CS2
		CLR		GLCD_DI
		SETB	GLCD_RW
GLCD_INIT2:
		SETB	GLCD_E
		MOV		A,GLCD_PORT
		CLR		GLCD_E
		JB		ACC.4,GLCD_INIT2	; Wait for right panel chip reset
		CLR		GLCD_CS2
; Clear screen
		CALL	GLCD_CLS
; Turn on left panel
		SETB	GLCD_CS1
		MOV		A,#00111111b		; Turn on LCD
		CALL	GLCD_WI
		MOV		A,#10111000b		; Set X-Page to 0
		CALL	GLCD_WI
		MOV		A,#01000000b		; Set Y-Address to 0
		CALL	GLCD_WI
		MOV		A,#11000000b		; Set display start line to 0
		CALL	GLCD_WI
		CLR		GLCD_CS1
; Turn on right panel
		SETB	GLCD_CS2
		MOV		A,#00111111b		; Turn on LCD
		CALL	GLCD_WI
		MOV		A,#10111000b		; Set X-page to 0
		CALL	GLCD_WI
		MOV		A,#01000000b		; Set Y-address to 0
		CALL	GLCD_WI
		MOV		A,#11000000b		; Set display start line to 0
		CALL	GLCD_WI
		CLR		GLCD_CS2
		RET
;
GLCD_CLS:
        PUSH    ACC
; Clear left panel
		SETB	GLCD_CS1
        MOV     XP,#10111000b
GLCD_CLS1:
		MOV		A,XP
		CALL	GLCD_WI
        MOV     YA,#01000000b
		MOV		A,YA
		CALL	GLCD_WI
GLCD_CLS2:
		MOV		A,#00h
		CALL	GLCD_WD
        INC     YA
        MOV     A,YA
        CJNE    A,#10000000b,GLCD_CLS2
        INC     XP
        MOV     A,XP
        CJNE    A,#11000000b,GLCD_CLS1
        CLR		GLCD_CS1
; Clear right panel
		SETB	GLCD_CS2
        MOV     XP,#10111000b
GLCD_CLS3:
		MOV		A,XP
		CALL	GLCD_WI
        MOV     YA,#01000000b
		MOV		A,YA
		CALL	GLCD_WI
GLCD_CLS4:
		MOV		A,#00h
		CALL	GLCD_WD
        INC     YA
        MOV     A,YA
        CJNE    A,#10000000b,GLCD_CLS4
        INC     XP
        MOV     A,XP
        CJNE    A,#11000000b,GLCD_CLS3
        CLR		GLCD_CS2
		POP     ACC
        RET
;
; Coordinate Transform
; X-Page, Y-Address, Mask calculation for given X, Y coordinates
GLCD_CT:
; Calculate X-Page
        MOV     A,X
        RR      A
        RR      A
        RR      A
        ORL     A,#10111000b
        ANL     A,#10111111b
        MOV     XP,A
; Calculate Chip and Y-Address
        MOV     A,Y
		CLR		C
		SUBB	A,#64
		JC		GLCD_CT4
        CLR     GLCD_CS1
		SETB	GLCD_CS2
		SJMP	GLCD_CT5
GLCD_CT4:
		MOV		A,Y
		SETB	GLCD_CS1
		CLR		GLCD_CS2
GLCD_CT5:
        ORL     A,#01000000b
        ANL     A,#01111111b
        MOV     YA,A
; Calculate bit mask
        MOV     A,X
		ANL		A,#00000111b
		MOV		R0,A
		MOV		A,#00000001b
GLCD_CT1:
        CJNE    R0,#00h,GLCD_CT2
		SJMP	GLCD_CT3
GLCD_CT2:
		RL		A
        DEC     R0
		SJMP	GLCD_CT1
GLCD_CT3:
        MOV     MASK,A
		RET
;
; Set a pixel
GLCD_SET:
        CALL    GLCD_CT
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        CALL    GLCD_RD		; Dummy read
        CALL    GLCD_RD		; Actual read
        MOV     VALUE,A
        MOV     A,MASK
        ORL     A,VALUE
        MOV     VALUE,A
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        MOV     A,VALUE
        CALL    GLCD_WD
        RET
;
; Clear a pixel
GLCD_CLR:
        CALL    GLCD_CT
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        CALL    GLCD_RD		; Dummy read
        CALL    GLCD_RD		; Actual read
        MOV     VALUE,A
        MOV     A,MASK
        XRL     A,#11111111b
        ANL     A,VALUE
        MOV     VALUE,A
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        MOV     A,VALUE
        CALL    GLCD_WD
        RET
;
; Negate a pixel
GLCD_XOR:
        CALL    GLCD_CT
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        CALL    GLCD_RD		; Dummy read
        CALL    GLCD_RD		; Actual read
        MOV     VALUE,A
        MOV     A,MASK
        XRL     A,VALUE
        MOV     VALUE,A
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        MOV     A,VALUE
        CALL    GLCD_WD
        RET
;
		END